@@ -37,12 +37,37 @@
 #include "xxhash.h"
 #include "pmurhash.h"
 
+#ifdef _WIN32
+#include <Windows.h>
+#endif
+
 unsigned long ZEXPORT digest_crc32(unsigned long crc,
                                    const unsigned char FAR *buf,
                                    unsigned len);
 
 static const char *sha2_hex_digits = "0123456789abcdef";
 
+FILE* open_with_widechar_on_windows(const char* txt) {
+    FILE* out;
+#ifdef _WIN32
+    wchar_t* buf;
+    size_t len = MultiByteToWideChar(CP_UTF8, 0, txt, -1, NULL, 0);
+    if (len <= 0) {
+        error("Cannot convert file to Unicode: %s", txt);
+    }
+    buf = (wchar_t*) R_alloc(len, sizeof(wchar_t));
+    if (buf == NULL) {
+        error("Could not allocate buffer of size: %ll", len);
+    }
+
+    MultiByteToWideChar(CP_UTF8, 0, txt, -1, buf, len);
+    out = _wfopen(buf, L"rb");
+#else
+    out = fopen(txt, "rb");
+#endif
+
+    return out;
+}
 
 SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Seed) {
     size_t BUF_SIZE = 1024;
@@ -67,6 +92,13 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
     } else { /* or a string */
         txt = (char*) STRING_VALUE(Txt);
         nChar = strlen(txt);
+
+        if (algo >= 100) {
+            fp = open_with_widechar_on_windows(txt);
+            if (!fp)  {
+              error("Cannot open input file: %s", txt);  /* #nocov */
+            }
+        }
     }
     if (skip > 0 && algo < 100) {
         if (skip>=nChar) {
@@ -189,9 +221,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         unsigned char md5sum[16];
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         md5_starts( &ctx );
         if (length>=0) {
@@ -205,7 +234,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
             while ( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0)
                 md5_update( &ctx, buf, nChar );
         }
-        fclose(fp);
         md5_finish( &ctx, md5sum );
         memcpy(output, md5sum, 16);
         if (!leaveRaw)
@@ -220,9 +248,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         unsigned char sha1sum[20];
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         sha1_starts ( &ctx );
         if (length>=0) {
@@ -235,7 +260,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
             while ( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0)
                 sha1_update( &ctx, buf, nChar );
         }
-        fclose(fp);
         sha1_finish ( &ctx, sha1sum );
         memcpy(output, sha1sum, 20);
         if (!leaveRaw)
@@ -247,9 +271,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         unsigned long val;
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         val  = digest_crc32(0L, 0, 0);
         if (length>=0) {
@@ -262,7 +283,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
             while ( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0)
                 val  = digest_crc32(val , buf, (unsigned) nChar);
         }
-        fclose(fp);
         sprintf(output, "%08x", (unsigned int) val);
         break;
     }
@@ -273,9 +293,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         unsigned char sha256sum[32];
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         sha256_starts ( &ctx );
         if (length>=0) {
@@ -288,7 +305,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
             while ( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0)
                 sha256_update( &ctx, buf, nChar );
         }
-        fclose(fp);
         sha256_finish ( &ctx, sha256sum );
         memcpy(output, sha256sum, 32);
         if (!leaveRaw)
@@ -304,9 +320,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
 
         unsigned char buf[BUF_SIZE];
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         SHA512_Init(&ctx);
         if (length>=0) {
@@ -319,7 +332,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
             while ( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0)
                 SHA512_Update( &ctx, buf, nChar );
         }
-        fclose(fp);
 
 		/* Calling SHA512_Final, because SHA512_End will already
 		   convert the hash to a string, and we also want RAW */
@@ -342,9 +354,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         XXH32_state_t* const state = XXH32_createState();
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         XXH_errorcode const resetResult = XXH32_reset(state, seed);
         if (resetResult == XXH_ERROR) {
@@ -367,7 +376,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
               }
             }
         }
-        fclose(fp);
         unsigned int val =  XXH32_digest(state);
         XXH32_freeState(state);
 
@@ -378,9 +386,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         XXH64_state_t* const state = XXH64_createState();
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         XXH_errorcode const resetResult = XXH64_reset(state, seed);
         if (resetResult == XXH_ERROR) {
@@ -403,7 +408,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
               }
             }
         }
-        fclose(fp);
         unsigned long long val =  XXH64_digest(state);
         XXH64_freeState(state);
 
@@ -419,9 +423,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
         size_t total_length = 0;
 
-        if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */
-        }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         if (length>=0) {
             while( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0
@@ -437,7 +438,6 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
                 total_length += nChar;
             }
         }
-        fclose(fp);
         unsigned int val = PMurHash32_Result(h1, carry, total_length);
 
         sprintf(output, "%08x", val);
@@ -448,6 +448,10 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
     }
     } /* end switch */
 
+    if (algo >= 100 && fp) {
+        fclose(fp);
+    }
+
     if (leaveRaw && output_length > 0) {
         PROTECT(result=allocVector(RAWSXP, output_length));
         memcpy(RAW(result), output, output_length);
